@@ -24,4 +24,4 @@ from .sym import sym
 @require_login
 def write_creds(options: GlobalOptions, resource: str, path: str, profile: str) -> None:
     """Write AWS credentials to a profile"""
-    options.create_saml_client(resource).write_creds(path_str=path, profile=profile)
+    options.create_saml_client(resource).write_creds(path=Path(path), profile=profile)
