@@ -2,10 +2,19 @@ import hashlib
 import json
 import os
 from contextlib import contextmanager
-from datetime import datetime
+from datetime import datetime, timedelta
 from pathlib import Path
 from tempfile import NamedTemporaryFile
-from typing import Any, Final, Iterator, Literal, MutableMapping, TypedDict, cast
+from typing import (
+    Any,
+    Final,
+    Iterator,
+    Literal,
+    MutableMapping,
+    Optional,
+    TypedDict,
+    cast,
+)
 
 import immutables
 import yaml
@@ -74,21 +83,26 @@ class SymConfigFile:
         with self as f:
             f.write(s)
 
+    def _throwaway_write(self):
+        with NamedTemporaryFile(mode="w+") as f:
+            yield f
+
     @contextmanager
     def exclusive_access(self):
         try:
             with self.write_lock as f:  # acquire write lock
                 yield f
         except AlreadyLocked:  # another thread is writing same value
-            with NamedTemporaryFile(mode="w") as f:  # throw away the input
-                yield f
+            yield from self._throwaway_write()
 
     @contextmanager
     def exclusive_create(self):
-        with self.exclusive_access() as f:
-            if f.buffer.peek():
-                raise AlreadyLocked()
-            yield f
+        self.mkparents()
+        if self.path.exists():
+            yield from self._throwaway_write()
+        else:
+            with self.exclusive_access() as f:
+                yield f
 
     @contextmanager
     def update(self):
@@ -171,6 +185,10 @@ class Config(MutableMapping[ConfigKey, Any]):
             cls.reset()
         return getattr(cls, "__instance")
 
+    @classmethod
+    def get_default(cls, key) -> Optional[str]:
+        return cls.instance().get(f"default_{key}")
+
     @classmethod
     def get_org(cls) -> str:
         return cls.instance()["org"]
@@ -188,8 +206,46 @@ class Config(MutableMapping[ConfigKey, Any]):
         return cls.get_servers().get(instance, ServerConfigSchema())
 
     @classmethod
-    def touch_instance(cls, instance: str, error: bool = False):
+    def find_instance_by_alias(
+        cls, alias: str, ttl: timedelta = timedelta(days=1)
+    ) -> Optional[str]:
+        for instance, config in cls.get_servers().items():
+            if alias in config.get("aliases", []) and cls.check_instance_ttl(
+                instance, ttl
+            ):
+                return instance
+
+    @classmethod
+    def check_instance_ttl(cls, instance: str, ttl: timedelta) -> Optional[str]:
+        instance_config = cls.get_instance(instance)
+        last_connect = instance_config.get("last_connection")
+        return last_connect and datetime.now() - last_connect < ttl
+
+    @classmethod
+    @contextmanager
+    def _atomic_instance_mutation(cls, instance: str):
         with cls.instance().atomic():
             instance_config = cls.get_instance(instance)
-            instance_config["last_connection"] = None if error else datetime.now()
+            yield instance_config
             cls.instance()["servers"] = cls.get_servers().set(instance, instance_config)
+
+    @classmethod
+    def touch_instance(cls, instance: str, error: bool = False):
+        with cls._atomic_instance_mutation(instance) as instance_config:
+            if error:
+                instance_config["last_connection"] = None
+                instance_config["aliases"] = []
+            else:
+                instance_config["last_connection"] = datetime.now()
+
+    @classmethod
+    def add_instance_alias(cls, instance: str, alias: str):
+        if (old_instance := cls.find_instance_by_alias(alias)) :
+            with cls._atomic_instance_mutation(old_instance) as instance_config:
+                instance_config["aliases"].remove(alias)
+
+        with cls._atomic_instance_mutation(instance) as instance_config:
+            if "aliases" not in instance_config:
+                instance_config["aliases"] = []
+            if alias not in instance_config["aliases"]:
+                instance_config["aliases"].append(alias)
