@@ -1,7 +1,7 @@
 import os
 import re
 import sys
-from datetime import datetime, timedelta
+from datetime import timedelta
 from pathlib import Path
 from signal import SIGPIPE, signal
 from subprocess import CalledProcessError
@@ -12,6 +12,7 @@ from ..decorators import intercept_errors, retry, run_subprocess
 from ..errors import (
     AccessDenied,
     FailedSubprocessError,
+    MissingPublicKey,
     SuppressedError,
     TargetNotConnected,
     WrappedSubprocessError,
@@ -19,6 +20,8 @@ from ..errors import (
 from ..helpers.boto import send_ssh_key
 from ..helpers.config import Config, SymConfigFile
 from ..helpers.params import get_ssh_user
+from ..helpers.sym_group import SymGroup
+from ..helpers.tee import Tee
 
 MissingPublicKeyPattern = re.compile(r"Permission denied \(.*publickey.*\)")
 TargetNotConnectedPattern = re.compile("TargetNotConnected")
@@ -53,11 +56,14 @@ def gen_ssh_key(dest: SymConfigFile):
 
 def ssh_args(client, instance, port) -> tuple:
     _, ssh_config = ssh_key_and_config(client)
-    return (
-        "ssh",
-        instance,
-        {"p": str(port), "F": str(ssh_config), "l": get_ssh_user(), "v": client.debug},
-    )
+    opts = {
+        "p": str(port),
+        "F": str(ssh_config),
+        "l": get_ssh_user(),
+        "v": client.debug,
+        "E": str(Tee.path_for_fd(client.log_dir, "ssh")) if client.log_dir else None,
+    }
+    return ("ssh", instance, opts)
 
 
 @run_subprocess
@@ -77,50 +83,62 @@ def _start_ssh_session(client: "SAMLClient", instance: str, port: int, *command:
     yield (*ssh_args(client, instance, port), *command)
 
 
-@retry(TargetNotConnected, delay=1)
+@intercept_errors(suppress=True)
+@run_subprocess
+def raw_ssh(*args):
+    yield ("ssh", *args)
+
+
+@retry(TargetNotConnected, delay=1, count=2)
+@retry(MissingPublicKey, delay=0, count=2)
 def start_ssh_session(
     client: "SAMLClient",
     instance: str,
     port: int,
+    *,
+    args: Sequence[str] = [],
     command: Sequence[str] = [],
-    retry=True,
+    wrap: bool = True,
 ):
-    ensure_ssh_key(client, instance, port)
+    ensure_ssh_key(client, instance, port, args=args)
     try:
-        _start_ssh_session(client, instance, port, *command)
+        _start_ssh_session(client, instance, port, *args, *command)
     except CalledProcessError as err:
+        if ConnectionClosedPattern.search(err.stderr):
+            raise SuppressedError(err, echo=True) from err
         client.dprint(f"SSH Session Error: {err.stderr}")
         Config.touch_instance(instance, error=True)
         if MissingPublicKeyPattern.search(err.stderr):
-            if retry:
-                start_ssh_session(client, instance, port, command, retry=False)
-            else:
-                raise WrappedSubprocessError(
-                    err, f"Does the user ({get_ssh_user()}) exist on the instance?"
-                ) from err
+            raise MissingPublicKey(err, get_ssh_user()) from err
         # If the ssh key path is cached then this doesn't get intercepted in ensure_ssh_key
         elif TargetNotConnectedPattern.search(err.stderr):
             raise TargetNotConnected() from err
         elif AccessDeniedPattern.search(err.stderr):
             raise AccessDenied() from err
-        elif ConnectionClosedPattern.search(err.stderr):
-            raise SuppressedError(err, echo=True) from err
         else:
-            raise WrappedSubprocessError(
-                err, f"Contact your Sym administrator.", report=True
-            ) from err
+            if wrap:
+                raise WrappedSubprocessError(
+                    err, f"Contact your Sym administrator.", report=True
+                ) from err
+            else:
+                raise SuppressedError(err, echo=True)
     else:
         Config.touch_instance(instance)
 
 
-@retry(TargetNotConnected, delay=1)
+@retry(TargetNotConnected, delay=1, count=2)
 @intercept_errors({TargetNotConnectedPattern: TargetNotConnected}, suppress=True)
-def ensure_ssh_key(client, instance: str, port: int, ttl=timedelta(days=1)):
+def ensure_ssh_key(
+    client,
+    instance: str,
+    port: int,
+    *,
+    ttl: timedelta = timedelta(days=1),
+    args: Sequence[str] = [],
+):
     ssh_key, ssh_config = ssh_key_and_config(client)
 
-    sym_cmd = (
-        f"sym {client.cli_options} ssh-session {client.resource} --instance %h --port %p"
-    )
+    sym_cmd = f"PYTHONUNBUFFERED=1 sym {client.cli_options} ssh-session {client.resource} --instance %h --port %p"
     # fmt: off
     ssh_config.put(dedent(  # Ensure the SSH Config first, always
         f"""
@@ -138,22 +156,19 @@ def ensure_ssh_key(client, instance: str, port: int, ttl=timedelta(days=1)):
     ))
     # fmt: on
 
-    instance_config = Config.get_instance(instance)
-
     if not ssh_key.path.exists():
         gen_ssh_key(ssh_key)
 
-    last_connect = instance_config.get("last_connection")
-    if last_connect and datetime.now() - last_connect < ttl:
+    if Config.check_instance_ttl(instance, ttl):
         client.dprint(f"Skipping remote SSH key check for {instance}")
         return
 
     try:
         _start_background_ssh_session(
-            client, instance, port, "exit", capture_output_=True
+            client, instance, port, *args, "exit", capture_output_=True
         )
     except CalledProcessError as err:
-        client.dprint(f"SSH Check Error: {err.stderr}")
+        client.dprint(f"SSH Check Error: {err.stdout}")
         if not MissingPublicKeyPattern.search(err.stderr):
             raise
         send_ssh_key(client, instance, ssh_key)
@@ -162,6 +177,8 @@ def ensure_ssh_key(client, instance: str, port: int, ttl=timedelta(days=1)):
 
 
 def start_tunnel(client, instance: str, port: int):
+    SymGroup.reset_tees()
+
     def handler(signum, frame):
         devnull = os.open(os.devnull, os.O_WRONLY)
         os.dup2(devnull, sys.stdout.fileno())
