@@ -55,6 +55,9 @@ class Tee(object):
         self.file.flush()
         self.fd.flush()
 
+    def fileno(self):
+        return self.fd.fileno()
+
     def __enter__(self):
         self.ref = self
 
