@@ -1,5 +1,6 @@
 import os
 from configparser import ConfigParser
+from datetime import datetime
 from pathlib import Path
 from typing import Final, Iterator, Tuple
 
@@ -10,7 +11,10 @@ from ..helpers.keywords_to_options import Argument
 from ..helpers.params import Profile
 from .saml_client import SAMLClient
 
-AwsCredentialsPath = Path(os.getenv('AWS_CREDENTIAL_FILE', Path.home() / '.aws' / 'credentials'))
+AwsCredentialsPath = Path(
+    os.getenv("AWS_CREDENTIAL_FILE", Path.home() / ".aws" / "credentials")
+)
+
 
 class AwsProfile(SAMLClient):
     binary = "aws"
@@ -19,7 +23,7 @@ class AwsProfile(SAMLClient):
     setup_help = f"Set up your profile in `{str(AwsCredentialsPath)}`."
 
     resource: str
-    options: 'GlobalOptions'
+    options: "GlobalOptions"
 
     @classmethod
     def validate_resource(cls, resource: str):
@@ -27,13 +31,16 @@ class AwsProfile(SAMLClient):
         config.read(AwsCredentialsPath)
         return config.has_section(resource)
 
-    def get_creds(self):
+    @property
+    def _section(self):
         config = ConfigParser()
         config.read(AwsCredentialsPath)
-        creds = {k.upper():v for k,v in config[self.resource].items() if k.startswith('aws')}
-        creds['AWS_REGION'] = config[self.resource].get('REGION')
-        return creds
+        return config[self.resource]
 
+    def get_creds(self):
+        creds = {k.upper(): v for k, v in self._section.items() if k.startswith("aws")}
+        creds["AWS_REGION"] = self._section.get("REGION")
+        return creds
 
     @intercept_errors()
     @run_subprocess
@@ -41,7 +48,7 @@ class AwsProfile(SAMLClient):
     def _exec(self, *args: str, **opts: str) -> Iterator[Tuple[Argument, ...]]:
         self.log_subprocess_event(args)
         with push_env("AWS_PROFILE", self.resource):
-            yield *args, opts
+            yield (*args, opts)
 
     def is_setup(self) -> bool:
         return AwsCredentialsPath.exists()
@@ -50,4 +57,8 @@ class AwsProfile(SAMLClient):
         return ConfigParser()
 
     def _ensure_session(self):
+        if (expires := self._section.get("x_security_token_expires")) :
+            parsed = datetime.fromisoformat(expires)
+            if parsed > datetime.now(parsed.tzinfo):
+                return
         get_identity(self)
