@@ -23,15 +23,17 @@ class SAMLClient(ABC):
     setup_help: ClassVar[Optional[str]] = None
 
     resource: str
-    debug: bool
+    options: "GlobalOptions"
+
     config_file: Final[SymConfigFile]
+
     _config: Optional[ConfigParser]
     _session_exists: bool
     _checked_setup: bool
 
-    def __init__(self, resource: str, *, debug: bool) -> None:
+    def __init__(self, resource: str, *, options: "GlobalOptions") -> None:
         self.resource = resource
-        self.debug = debug
+        self.options = options
 
         self._config = None
         self._session_exists = False
@@ -39,6 +41,14 @@ class SAMLClient(ABC):
 
         self.check_is_setup()
 
+    @property
+    def debug(self):
+        return self.options.debug
+
+    @property
+    def log_dir(self):
+        return self.options.log_dir
+
     @classmethod
     def validate_resource(cls, resource: str):
         return validations.validate_resource(resource)
@@ -74,16 +84,20 @@ class SAMLClient(ABC):
     def is_setup(self) -> bool:
         raise NotImplementedError
 
+    def _cli_options(self):
+        return {
+            "saml_client": self.option_value,
+            "debug": self.debug,
+            "log_dir": self.log_dir,
+        }
+
     @property
     def cli_options(self):
-        return shlex.join(
-            keywords_to_options(
-                [{"saml_client": self.option_value, "debug": self.debug,}]
-            )
-        )
+        return shlex.join(keywords_to_options([self._cli_options()]))
 
     def exec(self, *args: str, **kwargs: str) -> None:
         self.ensure_session()
+        self.dprint(f"client: {self.cli_options}")
         return self._exec(*args, **kwargs)
 
     def _ensure_session(self):
@@ -105,21 +119,16 @@ class SAMLClient(ABC):
 
     def subconfig(self, file_name, **deps):
         return SymConfigFile(
-            debug=self.debug,
-            saml_client_name=self.option_value,
-            resource=self.resource,
-            **deps,
-            file_name=file_name,
+            **self._cli_options(), resource=self.resource, **deps, file_name=file_name
         )
 
     def dprint(self, s: str):
-        if self.debug:
-            print(f"{s}\n")
+        self.options.dprint(s)
 
     def dconfig(self, config: ConfigParser):
         if self.debug:
-            print("Writing config:")
-            config.write(sys.stdout)
+            print("Writing config:", file=sys.stderr)
+            config.write(sys.stderr)
 
     def log_subprocess_event(self, command: tuple):
         segment.track("Subprocess Called", binary=command[0])
@@ -156,12 +165,14 @@ class SAMLClient(ABC):
             )
         }
 
-    def write_creds(self, *, path: str, profile: str):
+    def write_creds(self, *, path: Path, profile: str):
         creds = self.get_creds()
         creds["region"] = creds.pop("AWS_REGION", None)
         creds["x_security_token_expires"] = creds.pop("AWS_CREDENTIAL_EXPIRATION", None)
 
-        with read_write_lock(Path(path)) as f:
+        path.parent.mkdir(parents=True, exist_ok=True)
+
+        with read_write_lock(path) as f:
             config = ConfigParser()
             config.read_file(f)
 
@@ -178,6 +189,6 @@ class SAMLClient(ABC):
 
     def clone(self, *, klass: Type["SAMLClient"] = None, **overrides):
         kwargs = {
-            key: overrides.get(key, getattr(self, key)) for key in ["resource", "debug"]
+            key: overrides.get(key, getattr(self, key)) for key in ["resource", "options"]
         }
         return (klass or self.__class__)(**kwargs)
